require "vogogo_ruby/version"
require 'vogogo_ruby/exceptions/identifier_error'
require 'vogogo_ruby/exceptions/data_hash_error'
require 'httparty'
module VogogoRuby
	class VogogoDirectAPI
		include HTTParty
		API_URL = 'https://api.vogogo.com'
		URL_BASE = '/v1/'
		ACCOUNT = 'account'
		CUSTOMER = 'customer'
		INVOICE = 'invoice'
		ITEM = 'item'
		CUSTOMER_ACCOUNT = 'customer_account'
		CREDIT_CARD_CHARGE = 'charge'
		EFT_CREDIT = 'credit'
		DEPOSIT = 'deposit'
		WITHDRAWAL = 'withdrawal'

		# debug_output $stderr

		def initialize(key, token, user)
			@auth = {username: user, password: key + ':' + token}
		end

		#Retrieve a list of financial accounts
		def get_accounts
			list ACCOUNT
		end

		#Retrieve an account details
		def get_account account_id
			raise IdentifierError.new 'account_id cannot be empty' if account_id.nil? || account_id.empty?
			get ACCOUNT, account_id
		end

		def get_invoice invoice_id
			raise IdentifierError.new 'invoice_id cannot be empty' if invoice_id.nil? || invoice_id.empty?
			get INVOICE, invoice_id
		end

		def create_invoice data
			raise DataHashError.new 'invoice details hash cannot be empty' if data.nil? || data.empty?
			create INVOICE, data
		end

		def update_invoice invoice_id, data
			raise DataHashError.new 'invoice details hash cannot be empty' if data.nil? || data.empty?
			raise IdentifierError.new 'invoice_id cannot be empty' if invoice_id.nil? || invoice_id.empty?
			response = update INVOICE, invoice_id, data
		end

		def list_invoices
			list INVOICE
		end

		def delete_invoice invoice_id
			raise IdentifierError.new 'invoice_id cannot be empty' if invoice_id.nil? || invoice_id.empty?
			response = delete INVOICE, invoice_id
		end

		def create_item data
			raise DataHashError.new 'item details hash cannot be empty' if data.nil? || data.empty?
	    create ITEM, data
		end

		def delete_item item_id
			raise IdentifierError.new 'item_id cannot be empty' if item_id.nil? || item_id.empty?
			response = delete ITEM, item_id
		end

		def get_item item_id
			raise IdentifierError.new 'item_id cannot be empty' if item_id.nil? || item_id.empty?
			get ITEM, item_id
		end

		def list_items
			list ITEM
		end

		def update_item item_id, data
			raise DataHashError.new 'item details hash cannot be empty' if data.nil? || data.empty?
			raise IdentifierError.new 'item_id cannot be empty' if item_id.nil? || item_id.empty?
			response = update ITEM, item_id, data
		end

		def create_customer data
			raise DataHashError.new 'customer details hash cannot be empty' if data.nil? || data.empty?
			create CUSTOMER, data
		end

		def delete_customer customer_id
			raise IdentifierError.new 'customer_id cannot be empty' if customer_id.nil? || customer_id.empty?
			response = delete CUSTOMER, customer_id
		end

		def get_customer customer_id
			raise IdentifierError.new 'customer_id cannot be empty' if customer_id.nil? || customer_id.empty?
			get CUSTOMER, customer_id
		end

		def list_customers
			list CUSTOMER
		end

		def update_customer customer_id, data
			raise DataHashError.new 'customer details hash cannot be empty' if data.nil? || data.empty?
			raise IdentifierError.new 'customer_id cannot be empty' if customer_id.nil? || customer_id.empty?
			response = update CUSTOMER, customer_id, data
		end

		def create_customer_account data
			raise DataHashError.new 'customer account details hash cannot be empty' if data.nil? || data.empty?
	    create CUSTOMER_ACCOUNT, data
		end

		def delete_customer_account customer_account_id
			raise IdentifierError.new 'customer_account_id cannot be empty' if customer_account_id.nil? || customer_account_id.empty?
			response = delete CUSTOMER_ACCOUNT, customer_account_id
		end

		def get_customer_account customer_account_id
			raise IdentifierError.new 'customer_account_id cannot be empty' if customer_account_id.nil? || customer_account_id.empty?
			get CUSTOMER_ACCOUNT, customer_account_id
		end

		def list_customer_accounts
			list CUSTOMER_ACCOUNT
		end

		def post_credit_card_charge data
			raise DataHashError.new 'credit card charge details hash cannot be empty' if data.nil? || data.empty?
			create CREDIT_CARD_CHARGE, data
		end

		def post_eft_credit data
			raise DataHashError.new 'eft credit details hash cannot be empty' if data.nil? || data.empty?
			create EFT_CREDIT, data
		end

		def post_deposit data
			raise DataHashError.new 'deposit details hash cannot be empty' if data.nil? || data.empty?
			create DEPOSIT, data
		end

		def post_withdrawal data
			raise DataHashError.new 'withdrawal details hash cannot be empty' if data.nil? || data.empty?
			create WITHDRAWAL, data
		end

		private

		def list entity
			action('get', entity)
		end

		def create entity, data
			action('post', entity, '', data)
		end

		def get entity, id
			action('get', entity, id)
		end

		def update entity, id, data
			action('patch', entity, id, data)
		end

		def delete entity, id
			action('delete', entity, id)
		end

		def action method, entity, id = '', data = {}
			options = {body: data.to_json, basic_auth: auth, headers: {'Content-Type' => 'application/json'}}
			id << '/' unless id.empty?
	 		self.class.send(method, path(entity.to_s + '/' + id.to_s), options)
		end

		def auth
			@auth
		end

		def path(method)
			API_URL + URL_BASE + method
		end

	end
end
